@@ -24,7 +24,7 @@
 #' @param check whether to check the soil profile (default is TRUE)
 #' @param physical whether soil physical properties are obtained from the data base or through \sQuote{SR}, Saxton and Rawls pedotransfer functions.
 #' @param xargs additional arguments passed to \code{\link{apsimx_soil_profile}} or \sQuote{apsimx:::approx_soil_variable} function. At the moment these are:
-#' \sQuote{soil.bottom}, \sQuote{method} and \sQuote{nlayers}.
+#' \sQuote{soil.bottom}, \sQuote{crops}, and \sQuote{nlayers} for the first function and \sQuote{method} for the second function. 
 #' @return it generates an object of class \sQuote{soil_profile}.
 #' @details Variable which are directly retrieved and a simple unit conversion is performed: \cr
 #' * Bulk density - bdod \cr
@@ -120,25 +120,36 @@ get_isric_soil_profile <- function(lonlat,
   if(any(is.na(soc))) stop("No soil data available for this location. Did you specify the coordinates correctly?")
 
   ## These are the default thicknesses in ISRIC
+  o.thcknss <- c(50, 100, 150, 300, 400, 1000) ## in mm
   thcknss <- c(50, 100, 150, 300, 400, 1000) ## in mm
   
   ## Some variables can be passed to apsimx:::approx_soil_variable
   soil.bottom <- 200
   method <- "constant"
-  nlayers <- 10
+  nlayers <- 6
   crps <- c("Maize", "Soybean", "Wheat")
   if(!is.null(xargs)){
     ### Soil bottom
-    if(!is.null(xargs$soil.bottom)){
+    if(!is.null(xargs$soil.bottom) && is.null(xargs$nlayers)){
       soil.bottom <- xargs$soil.bottom
+      thcknss <- apsimx_soil_profile(nlayers = nlayers, soil.bottom = soil.bottom)$soil$Thickness
     }
     ### Method
     if(!is.null(xargs$method)){
       method <- xargs$method
     }
     ### Number of layers
-    if(!is.null(xargs$nlayers)){
+    if(!is.null(xargs$nlayers) && is.null(xargs$soil.bottom)){
       nlayers <- xargs$nlayers
+      ### Need to redefine 'Thickness'
+      thcknss <- apsimx_soil_profile(nlayers = nlayers)$soil$Thickness
+    }
+    ### Do I need this?
+    if(!is.null(xargs$nlayers) && !is.null(xargs$soil.bottom)){
+      nlayers <- xargs$nlayers
+      soil.bottom <- xargs$soil.bottom
+      ### Need to redefine 'Thickness'
+      thcknss <- apsimx_soil_profile(nlayers = nlayers, soil.bottom = soil.bottom)$soil$Thickness
     }
     ### Crops
     if(!is.null(xargs$crops)){
@@ -148,8 +159,12 @@ get_isric_soil_profile <- function(lonlat,
 
   ## Create the empty soil profile
   if(missing(soil.profile)){
-    new.soil <- FALSE
-    soil_profile <- apsimx_soil_profile(nlayers = 6, Thickness = thcknss, crops = crps) 
+    if(is.null(xargs)){
+      new.soil <- FALSE
+    }else{
+      new.soil <- TRUE      
+    }
+    soil_profile <- apsimx_soil_profile(nlayers = nlayers, Thickness = thcknss, soil.bottom = soil.bottom, crops = crps) 
     soil_profile$soil$ParticleSizeClay <- NA
     soil_profile$soil$ParticleSizeSilt <- NA
     soil_profile$soil$ParticleSizeSand <- NA
@@ -168,25 +183,51 @@ get_isric_soil_profile <- function(lonlat,
   ### For some of the conversions see: https://www.isric.org/explore/soilgrids/faq-soilgrids
   if(new.soil){
     sp.xout <- cumsum(soil_profile$soil$Thickness)
-    soil_profile$soil$BD <- approx_soil_variable(data.frame(x = cumsum(thcknss), y = bdod[[1]] * 1e-2), 
-                                                 xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y    
-    soil_profile$soil$Carbon <- approx_soil_variable(data.frame(x = cumsum(thcknss), y = soc[[1]] * 1e-2), 
-                                                 xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y    
-    soil_profile$soil$PH <- approx_soil_variable(data.frame(x = cumsum(thcknss), y = phh2o[[1]] * 1e-1), 
-                                                     xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y    
-    soil_profile$soil$ParticleSizeClay <- approx_soil_variable(data.frame(x = cumsum(thcknss), y = clay[[1]] * 1e-1), 
-                                                 xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y   
-    soil_profile$soil$ParticleSizeSand <- approx_soil_variable(data.frame(x = cumsum(thcknss), y = sand[[1]] * 1e-1), 
+    
+    ### Will this take care of both number of layers and soil.bottom?
+    if(length(cumsum(thcknss)) != length(bdod[[1]])){
+      bdod.dat <- stats::approx(data.frame(cumsum(o.thcknss), bdod[[1]] * 1e-2), xout = sp.xout)
+      soc.dat <- stats::approx(data.frame(cumsum(o.thcknss), soc[[1]] * 1e-2), xout = sp.xout)
+      phh2o.dat <- stats::approx(data.frame(cumsum(o.thcknss), phh2o[[1]] * 1e-1), xout = sp.xout)
+      clay.dat <- stats::approx(data.frame(cumsum(o.thcknss), clay[[1]] * 1e-1), xout = sp.xout)
+      sand.dat <- stats::approx(data.frame(cumsum(o.thcknss), sand[[1]] * 1e-1), xout = sp.xout)
+      nitrogen.dat <- stats::approx(data.frame(cumsum(o.thcknss), nitrogen[[1]]), xout = sp.xout)
+      cec.dat <- stats::approx(data.frame(cumsum(o.thcknss), cec[[1]]), xout = sp.xout)
+      wv0010.dat <- stats::approx(data.frame(cumsum(o.thcknss), wv0010[[1]]), xout = sp.xout)
+      wv0033.dat <- stats::approx(data.frame(cumsum(o.thcknss), wv0033[[1]]), xout = sp.xout)
+      wv1500.dat <- stats::approx(data.frame(cumsum(o.thcknss), wv1500[[1]]), xout = sp.xout)
+    }else{
+      bdod.dat <- data.frame(x = cumsum(thcknss), y = bdod[[1]] * 1e-2)
+      soc.dat <- data.frame(x = cumsum(thcknss), y = soc[[1]] * 1e-2)
+      phh2o.dat <- data.frame(x = cumsum(thcknss), y = phh2o[[1]] * 1e-1)
+      clay.dat <- data.frame(x = cumsum(thcknss), y = clay[[1]] * 1e-1)
+      sand.dat <- data.frame(x = cumsum(thcknss), y = sand[[1]] * 1e-1)
+      nitrogen.dat <- data.frame(x = cumsum(thcknss), y = nitrogen[[1]])
+      cec.dat <- data.frame(x = cumsum(thcknss), y = cec[[1]])
+      wv0010.dat <- data.frame(x = cumsum(thcknss), y = wv0010[[1]])
+      wv0033.dat <- data.frame(x = cumsum(thcknss), y = wv0033[[1]])
+      wv1500.dat <- data.frame(x = cumsum(thcknss), y = wv1500[[1]])
+    }
+
+    soil_profile$soil$BD <- approx_soil_variable(bdod.dat, 
+                                                 xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y
+    soil_profile$soil$Carbon <- approx_soil_variable(soc.dat, 
+                                                 xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y
+    soil_profile$soil$PH <- approx_soil_variable(phh2o.dat, 
+                                                     xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y 
+    soil_profile$soil$ParticleSizeClay <- approx_soil_variable(clay.dat, 
+                                                 xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y 
+    soil_profile$soil$ParticleSizeSand <- approx_soil_variable(sand.dat, 
                                                    xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y 
-    soil_profile$soil$Nitrogen <- approx_soil_variable(data.frame(x = cumsum(thcknss), y = nitrogen[[1]]), 
-                                                   xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y   
-    soil_profile$soil$CEC <- approx_soil_variable(data.frame(x = cumsum(thcknss), y = cec[[1]]), 
-                                                       xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y   
-    soil_profile$soil$SAT <- approx_soil_variable(data.frame(x = cumsum(thcknss), y = wv0010[[1]]), 
+    soil_profile$soil$Nitrogen <- approx_soil_variable(nitrogen.dat, 
+                                                   xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y
+    soil_profile$soil$CEC <- approx_soil_variable(cec.dat, 
+                                                       xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y 
+    soil_profile$soil$SAT <- approx_soil_variable(wv0010.dat, 
                                                      xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y * 1e-3
-    soil_profile$soil$DUL <- approx_soil_variable(data.frame(x = cumsum(thcknss), y = wv0033[[1]]), 
+    soil_profile$soil$DUL <- approx_soil_variable(wv0033.dat, 
                                                      xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y * 1e-3  
-    soil_profile$soil$LL15 <- approx_soil_variable(data.frame(x = cumsum(thcknss), y = wv1500[[1]]), 
+    soil_profile$soil$LL15 <- approx_soil_variable(wv1500.dat, 
                                                   xout = sp.xout, soil.bottom = soil.bottom, method = method, nlayers = nlayers)$y * 1e-3   
   }else{
     soil_profile$soil$BD <- bdod[[1]] * 1e-2
@@ -210,9 +251,8 @@ get_isric_soil_profile <- function(lonlat,
     soil_profile$soil$SAT <- sr_sat(soil_profile$soil$ParticleSizeSand, soil_profile$soil$DUL, DUL_S)    
   }
 
-  B <- (log(1500) - log(33))/(log(soil_profile$soil$DUL) - log(soil_profile$soil$LL15))
-  Lambda <- 1/B
-  soil_profile$soil$KS <- (1930 * (soil_profile$soil$SAT - soil_profile$soil$DUL)^(3 - Lambda)) * 100
+  ## Comparing this to the previous calculation
+  soil_profile$soil$KS <- sr_ks(soil_profile$soil$ParticleSizeClay, soil_profile$soil$ParticleSizeSand, soil_profile$soil$Carbon * 2)
   
   soil_profile$soil$AirDry <- soil_profile$soil$LL15
   soil_profile$soil$AirDry[1] <- soil_profile$soil$LL15[1] * 0.5 ## AirDry is half of LL for the first layer
@@ -285,7 +325,7 @@ get_isric_soil_profile <- function(lonlat,
                           "- geomdesc =", NA)
   
   soil_profile$metadata <- alist
-  
+
   if(fix) soil_profile <- fix_apsimx_soil_profile(soil_profile, verbose = verbose)
   
   if(check) check_apsimx_soil_profile(soil_profile)
@@ -297,10 +337,14 @@ get_isric_soil_profile <- function(lonlat,
 #### Pedo Transfer equations (Saxton and Rawls) ####
 
 ## Field Capacity or DUL
+## According to Saxton and Rawls the inputs are in percent
+## However, to match the values in Table 3 I need to convert
+## clay and sand to proportion, BUT OM stays in percent!
 sr_dul <- function(clay, sand, om){
-  clay <- clay * 1e-2
-  sand <- sand * 1e-2
-  om <- om * 1e-2
+  p2p <- 1e-2
+  clay <- clay * p2p
+  sand <- sand * p2p
+  om <- om 
   ans0 <- -0.251 * sand + 0.195 * clay + 0.011 * om +
     0.006 * (sand * om) - 0.027 * (clay * om) + 0.452 * (sand * clay) + 0.299
   ans <- ans0 + (1.283 * ans0^2 - 0.374 * ans0 - 0.015)
@@ -308,9 +352,10 @@ sr_dul <- function(clay, sand, om){
 }
 
 sr_dul_s <- function(clay, sand, om){
-  clay <- clay * 1e-2
-  sand <- sand * 1e-2
-  om <- om * 1e-2
+  p2p <- 1e-2
+  clay <- clay * p2p
+  sand <- sand * p2p
+  om <- om 
   ans0 <- 0.278 * sand + clay * 0.034 + om * 0.022 +
     -0.018 * sand * om - 0.027 * clay * om + 
     -0.584 * sand * clay + 0.078
@@ -325,18 +370,47 @@ sr_sat <- function(sand, sr_dul, sr_dul_s){
 }
 
 sr_ll <- function(clay, sand, om){
-  clay <- clay * 1e-2
-  sand <- sand * 1e-2
-  om <- om * 1e-2
+  p2p <- 1e-2 ## percent to proportion
+  clay <- clay * p2p
+  sand <- sand * p2p
+  om <- om
   ans0 <- -0.024 * sand + 0.487 * clay + 0.006 * om + 
-    0.005 * sand * om + 0.013 *clay * om + 0.068 *sand * clay +  0.031
+    0.005 * (sand * om) - 0.013 * (clay * om) + 0.068 * (sand * clay) + 0.031
   ans <- ans0 + (0.14 * ans0 - 0.02)
   ans
 } 
 
+### Saxton and Rawls KS
+### This version was developed to estimate KS in mm/h
+### and this is for 'gravel-free' soil
+### Multiplying by 24 converts hours to a full day
+### but I'm not sure this will work. APSIM is not too
+### sensitive to this soil property as long as it is not
+### too low
+sr_ks <- function(clay, sand, om, units = c("mm/day", "mm/h")){
+  
+  units <- match.arg(units)
+  dul <- sr_dul(clay, sand, om)
+  dul_s <- sr_dul_s(clay, sand, om)
+  ll15 <- sr_ll(clay, sand, om)
+  sat <- sr_sat(sand, dul, dul_s)
+  B <- (log(1500) - log(33))/(log(dul) - log(ll15))
+  Lambda <- 1/B
+  conv <- ifelse(units == "mm/day", 24, 1)
+  ans <- 1930 * (sat - dul)^(3 - Lambda) * conv
+  
+  if(any(is.na(ans))){
+    stop("The function resulted in an NA")
+  }
+  ans
+}
+
 ## Texture to other parameters
 texture2soilParms <- function(texture.class = "NO DATA"){
 
+  if(length(texture.class) > 1)
+    stop("This function only takes arguments of length equal to 1", call. = FALSE)
+  
   textureClasses <- c("clay", "silty clay", "sandy clay", "clay loam", "silty clay loam", "sandy clay loam", "loam", "silty loam", "sandy loam", "silt", "loamy sand", "sand", "NO DATA")  
   Albedo <- c(0.12, 0.12, 0.13, 0.13, 0.12, 0.13, 0.13, 0.14, 0.13, 0.13, 0.16, 0.19, 0.13)
   CN2 <- c(73.0, 73.0, 73.0, 73.0, 73.0, 73.0, 73.0, 73.0, 68.0, 73.0, 68.0, 68.0, 73.0)
@@ -372,6 +446,9 @@ intl_sand_propn <- function( usda_clay, usda_silt ) {
 
 texture_class <- function (usda_clay, usda_silt ) {
   
+  if(length(usda_clay) > 1 || length(usda_silt) > 1)
+    stop("This function only accepts numeric vectors of length equal to 1", call. = FALSE)
+  
   if(usda_clay < 0 || usda_clay > 1) stop("usda_clay should be between 0 and 1")
   if(usda_silt < 0 || usda_silt > 1) stop("usda_silt should be between 0 and 1")
   
@@ -405,3 +482,69 @@ texture_class <- function (usda_clay, usda_silt ) {
   
   return( class )
 }
+
+### Saxton and Rawls (Table 3 in the paper)
+sr_table <- function(clay, sand, om){
+  
+  if(missing(clay)){
+    texture.classes <- c("sand", "loamy sand", "sandy loam", "loam", "silty loam", "silt", "sandy clay loam", "clay loam", "silty clay loam", "silty clay", "sandy clay", "clay")
+  }else{
+    
+    if(length(clay) != length(sand))
+      stop("length of 'clay' should be the same as length of 'sand'")
+    
+    if(length(clay) != length(om))
+      stop("length of 'clay' should be the same as length of 'om'")
+    
+    if(any(clay < 0)) stop("Clay should be greater than zero")
+    if(any(clay > 100)) stop("Clay should be less than 100")
+    if(any(sand < 0)) stop("Sand should be greater than zero")
+    if(any(sand > 100)) stop("Sand should be less than 100")
+    
+  }
+  
+  ans <- data.frame(texture.class = NA, sand = NA, clay = NA,
+                    om = NA, ll15 = NA, dul = NA, sat = NA, 
+                    paw = NA, ks = NA)
+  
+  if(missing(clay)){
+    clay <- c(5, 5, 10, 20, 15, 5, 25, 35, 35, 45, 40, 50)
+    sand <- c(88, 80, 65, 40, 20, 10, 60, 30, 10, 10, 50, 25)
+    om <- rep(2.5, length(clay))
+    for(i in seq_along(texture.classes)){
+      ans[i, "texture.class"] <- texture.classes[i]
+      ans[i, "sand"] <- sand[i]
+      ans[i, "clay"] <- clay[i]
+      ans[i, "om"] <- om[i]
+      ll15 <- sr_ll(clay[i], sand[i], om[i])
+      ans[i, "ll15"] <- ll15
+      dul <- sr_dul(clay[i], sand[i], om[i])
+      ans[i, "dul"] <- dul
+      dul_s <- sr_dul_s(clay[i], sand[i], om[i])
+      ans[i, "sat"] <- sr_sat(sand[i], dul, dul_s)
+      ans[i, "paw"] <- dul - ll15 
+      ans[i, "ks"] <- sr_ks(clay[i], sand[i], om[i], units = "mm/h")
+    }    
+  }else{
+    for(i in seq_along(clay)){
+      clay.plus.sand <- clay[i] + sand[i]
+      if(clay.plus.sand >= 100){
+        warning("Clay plus sand cannot be greater or equal to 100")
+      }
+      silt <- 100 - (clay[i] + sand[i])
+      ans[i, "texture.class"] <- texture_class(clay[i], silt)
+      ans[i, "sand"] <- sand[i]
+      ans[i, "clay"] <- clay[i]
+      ans[i, "om"] <- om[i]
+      ll15 <- sr_ll(clay[i], sand[i], om[i])
+      ans[i, "ll15"] <- ll15
+      dul <- sr_dul(clay[i], sand[i], om[i])
+      ans[i, "dul"] <- dul
+      dul_s <- sr_dul_s(clay[i], sand[i], om[i])
+      ans[i, "sat"] <- sr_sat(sand[i], dul, dul_s)
+      ans[i, "paw"] <- dul - ll15 
+      ans[i, "ks"] <- sr_ks(clay[i], sand[i], om[i], units = "mm/h")
+    }
+  }
+ return(ans) 
+}
\ No newline at end of file
