@@ -6,7 +6,7 @@
 #' This option is useful when the intention is to show the simulation structure to pick a root presumably. \sQuote{parm}
 #' can be set as 0, 1, 2 or 3 for different levels.
 #' \sQuote{parm} can also be a list with integers, such as \sQuote{list(1, 2, 3)}. If zero is included, available elements
-#  will be displayed.
+#  will be displayed. At the moment, the maximum length of this list is 7.
 #' If a parameter is specified the function will try to \sQuote{guess} the root elements from the parameter path supplied.
 #'
 #' @title Inspect an .apsimx (JSON) file
@@ -87,7 +87,10 @@ inspect_apsimx <- function(file = "", src.dir = ".",
                            print.path = FALSE,
                            root){
   #### Beginning of function ----
-  ##.check_apsim_name(file)
+  if(isFALSE(get("allow.path.spaces", envir = apsimx::apsimx.options))){
+    .check_apsim_name(file)
+    .check_apsim_name(normalizePath(src.dir))
+  }
 
   file.names <- dir(path = src.dir, pattern=".apsimx$", ignore.case=TRUE)
 
@@ -383,11 +386,29 @@ inspect_apsimx <- function(file = "", src.dir = ".",
   ## 'Models.Core.Zone'
   if(find.root){
     wcz <- grepl("Models.Core.Zone", parent.node)
-    if(sum(wcz) < 0.5)
-      stop("Core Zone Simulation not found", call. = FALSE)
-    core.zone.node <- parent.node[wcz][[1]]$Children
 
-    parm.path.2 <- paste0(parm.path.1, ".", parent.node[wcz][[1]]$Name)
+    ## sapply(parent.node, \(x) x$`$type`)
+    ## browser()
+    ### In case of multiple fields per simulation, 'Models.Core.Zone' does not exist
+    ### FIXME: The code below needs more careful thinking about how I extract "Models.Core.Zone"
+    ### and what does the [[1]] mean below
+    if(sum(wcz) < 0.5){
+      wcfz <- grepl("Models.* Models", parent.node)  ## This is for which core field zone
+      if(all(wcfz)){
+        core.zone.node <- parent.node
+        parm.path.2 <- parm.path.1 
+      }else{
+        stop("Core Zone Simulation not found", call. = FALSE)
+      }
+    }else{
+      ### What does the [[1]] imply here?
+      core.zone.node <- parent.node[wcz][[1]]$Children
+      parm.path.2 <- paste0(parm.path.1, ".", parent.node[wcz][[1]]$Name)
+    }
+    
+    if(is.null(core.zone.node))
+      stop("Core Zone Simulation not found", call. = FALSE)
+
   }
 
   if(node == "Soil"){
@@ -431,6 +452,7 @@ inspect_apsimx <- function(file = "", src.dir = ".",
     }else{
       ## Pick which soil component we want to look at
       ## Which is not 'Metadata"
+      ## browser()
       wsiw <- "InitialWater" %in% soil.children.names
       if(soil.child %in% c("Water", "InitialWater")){
         if(soil.child == "InitialWater" && isFALSE(wsiw)){
@@ -442,10 +464,10 @@ inspect_apsimx <- function(file = "", src.dir = ".",
             if(length(wsiw3) > 0){
               soil.child <- "Water"
             }else{
-              soil.child <- "InitialWater"                          
+              soil.child <- "InitialWater"
             }
           }
-        }        
+        }
       }
 
       if(soil.child != "Solute"){
@@ -509,6 +531,7 @@ inspect_apsimx <- function(file = "", src.dir = ".",
         }
         if(length(tmp) > 1){
           if(!ii %in% c("Crop LL", "Crop KL", "Crop XF")){
+            if(grepl("Metadata", ii)) next
             col.nms <- c(col.nms, ii)
             vals <- as.vector(unlist(tmp))
             if(is.null(vals)) vals <- rep(".", length(tmp))
@@ -592,10 +615,10 @@ inspect_apsimx <- function(file = "", src.dir = ".",
     ssnc <- selected.soil.node.child
 
     if(soil.child %in% c("Solute", "NO3", "NH4", "Urea")){
-      
+
       solutes <- sapply(ssnc, function(x) x$Name)
       parm.path <- paste0(parm.path.2.1, ".", soil.child)
-      
+
       if(missing(parm)){
         cat("Solutes:", solutes, "\n")
       }else{
@@ -603,11 +626,11 @@ inspect_apsimx <- function(file = "", src.dir = ".",
           sel.parm <- grep(parm[[1]], solutes)
           if(length(sel.parm) == 0)
             stop("'parm' should be one of: ", paste(solutes, collapse = ", "), call. = FALSE)
-          solutes <- parm[[1]]          
+          solutes <- parm[[1]]
         }
         if(soil.child %in% c("NO3", "NH4", "Urea")){
           ## In this case 'parm' could be either Thickness, InitialValues
-          ## or some of the other ones. 
+          ## or some of the other ones.
           parm.path <- paste0(parm.path.2.1, ".", soil.child)
         }
       }
@@ -623,22 +646,22 @@ inspect_apsimx <- function(file = "", src.dir = ".",
         soil.d1 <- data.frame(Thickness = unlist(ssnc.solute$Thickness),
                               InitialValues = unlist(ssnc.solute$InitialValues))
         if(missing(parm)){
-          if(!is.null(soil.d1)) print(knitr::kable(soil.d1, digits = digits))  
+          if(!is.null(soil.d1)) print(knitr::kable(soil.d1, digits = digits))
         }else{
           if(length(parm) == 1){
             if(parm %in% c("Thickness", "InitialValues")){
               soil.d1.s <- soil.d1[, parm, drop = FALSE]
-              if(!is.null(soil.d1)) print(knitr::kable(soil.d1.s, digits = digits))  
+              if(!is.null(soil.d1)) print(knitr::kable(soil.d1.s, digits = digits))
             }else{
               ### Parm is of length equal to 1 and is not "Thickness" or "InitialValues"
               if(parm %in% c("NO3", "NH4", "Urea")){
-                if(!is.null(soil.d1)) print(knitr::kable(soil.d1, digits = digits))  
+                if(!is.null(soil.d1)) print(knitr::kable(soil.d1, digits = digits))
               }
-            }          
-          }          
+            }
+          }
         }
-          
-        cnms <- setdiff(names(ssnc.solute), enms)        
+
+        cnms <- setdiff(names(ssnc.solute), enms)
         for(k in cnms){
           if(k %in% c("Thickness", "InitialValues")) next
           tmp <- ssnc.solute[k][[1]]
@@ -653,13 +676,13 @@ inspect_apsimx <- function(file = "", src.dir = ".",
             if(soil.child == "Solute"){
               if(!parm %in% solutes)
                 stop("'parm' should be one of:", solutes, call. = FALSE)
-              if(!is.null(soil.d2)) print(knitr::kable(soil.d2, digits = digits))                          
+              if(!is.null(soil.d2)) print(knitr::kable(soil.d2, digits = digits))
             }else{
               if(!parm %in% c("Thickness", "InitialValues")){
                 if(!parm %in% soil.d2$parm)
                   stop("'parm' should be one of:", paste(soil.d2$parm, collapse = " "), call. = FALSE)
-                soil.d2.s <- soil.d2[soil.d2$parm == parm, , drop = FALSE] 
-                if(!is.null(soil.d2)) print(knitr::kable(soil.d2.s, digits = digits)) 
+                soil.d2.s <- soil.d2[soil.d2$parm == parm, , drop = FALSE]
+                if(!is.null(soil.d2)) print(knitr::kable(soil.d2.s, digits = digits))
               }
             }
           }
@@ -669,14 +692,14 @@ inspect_apsimx <- function(file = "", src.dir = ".",
             if(is.numeric(parm[[2]])){
               if(parm[[2]] < 1 || parm[[2]] > nrow(soil.d2))
                 stop("The second elment of 'parm' should be a number between 1 and ", nrow(soil.d2), call. = FALSE)
-              soil.d2.s <- soil.d2[parm[[2]], , drop = FALSE]  
+              soil.d2.s <- soil.d2[parm[[2]], , drop = FALSE]
             }
             if(is.character(parm[[2]])){
               if(!parm[[2]] %in% soil.d2$parm)
                 stop("The second element of 'parm' should be one of:", soil.d2$parm, call. = FALSE)
-              soil.d2.s <- soil.d2[soil.d2$parm == parm[[2]], , drop = FALSE]  
+              soil.d2.s <- soil.d2[soil.d2$parm == parm[[2]], , drop = FALSE]
             }
-            if(!is.null(soil.d2)) print(knitr::kable(soil.d2.s, digits = digits))            
+            if(!is.null(soil.d2)) print(knitr::kable(soil.d2.s, digits = digits))
           }
         }
         soil.d2 <- NULL
@@ -1176,8 +1199,45 @@ inspect_apsimx <- function(file = "", src.dir = ".",
             print(knitr::kable(pdat[parm[[6]],, drop = FALSE], row.names = TRUE))
           }
         }
-        if(length(parm) >= 7)
+        if(length(parm) >= 7){
+
+          names.level.6 <- FALSE
+          root.names.level.6 <- vapply(wlevel5$Children, FUN = function(x) x$Name,
+                                       FUN.VALUE = "character")
+          if(length(root.names.level.6) == 0){
+            root.names.level.6 <- names(wlevel5)
+            names.level.6 <- TRUE
+          }
+          if(length(parm[[7]]) == 1){
+            if(parm[[7]] > 0){
+              if(names.level.6){
+                selected.names <- root.names.level.6[parm[[7]]]
+                ##pdat <- data.frame(sixth_level = selected.names)
+                ##print(knitr::kable(pdat, row.names = TRUE))
+                parm.path <- paste0(parm.path, ".", selected.names)
+              }else{
+                wlevel6 <- try(apsimx_json$Children[[parm[[2]]]]$Children[[parm[[3]]]]$Children[[parm[[4]]]]$Children[[parm[[5]]]]$Children[[parm[[6]]]]$Children[[parm[[7]]]], silent = TRUE)
+                if(inherits(wlevel6, "try-error"))
+                  stop("parm[[7]] did not match an available node", call. = FALSE)
+                parm.path <- paste0(parm.path, ".", root.names.level.6[parm[[7]]])
+              }
+            }else{
+              pdat <- data.frame(seventh_level = root.names.level.6)
+              print(knitr::kable(pdat, row.names = TRUE))
+            }
+          }else{
+            ### Here I assume that parm[[7]] is a vector
+            if(length(parm[[7]]) > length(root.names.level.6))
+              stop("Length of parm[[7]] should be less than number of elements in the seventh level")
+            selected.levels <- root.names.level.6[parm[[7]]]
+            parm.path <- paste0(parm.path, ".", selected.levels)
+            pdat <- data.frame(seventh_level = root.names.level.6)
+            print(knitr::kable(pdat[parm[[7]],, drop = FALSE], row.names = TRUE))
+          }
+        }
+        if(length(parm) >= 8){
           stop("Have not developed this yet", call. = FALSE)
+        }
       }else{
         ## This is if the parm is just one term
         parm.found <- FALSE
@@ -1671,3 +1731,16 @@ parse_root <- function(root){
   }
   root
 }
+
+### This function is from ChatGPT
+traverse_json <- function(file, path = "$") {
+  
+  json_obj <- jsonlite::read_json(file)
+  if (is.list(json_obj)) {
+    for (key in names(json_obj)) {
+      traverse_json(json_obj[[key]], paste0(path, ".", key))
+    }
+  } else {
+    print(paste(path, ":", json_obj))
+  }
+}
