@@ -1,19 +1,25 @@
 #'
 #' It is possible to provide a list of soil profiles for replacement in the simulations. In this
-#' case, the parameter path can be simply \sQuote{soil.profile} or \sQuote{soil_profile} if there 
+#' case, the parameter path can be simply \sQuote{soil.profile} or \sQuote{soil_profile} if there
 #' is one single simulation. It can also be the path to \sQuote{Soil}. In this case, the path should
 #' be something such as \sQuote{Simulations.SimulationName.Soil}. \sQuote{SimulationName} should
 #' be replaced with the appropriate string.
-#' 
-#' In the grid, the column with name \sQuote{soil.profile} should contain integers that will be used to 
-#' pick from the list of provided soil profiles. In this case it is possible to re-use them. 
-#' For example, the values could be 1, 2, 3, etc. to select the corresponding soil profiles from 
+#'
+#' In the grid, the column with name \sQuote{soil.profile} should contain integers that will be used to
+#' pick from the list of provided soil profiles. In this case it is possible to re-use them.
+#' For example, the values could be 1, 2, 3, etc. to select the corresponding soil profiles from
 #' the \sQuote{soil.profiles} list.
-#' 
+#'
 #' If the \sQuote{cores} argument is greater than 1, then the package \CRANpkg{future} is required.
 #' It will first search for a future plan under options and if nothing is found it will chose an OS-appropriate plan
-#' and it uses the chosen number of cores for execution. Errors, messages and warnings are normally suppressed 
-#' during parallel execution, so it is important to ensure that the simulations are constructed properly.
+#' (\sQuote{multisession}) and it uses the chosen number of cores for execution. Errors, messages and warnings are normally 
+#' suppressed during parallel execution, so it is important to ensure that the simulations are constructed properly.
+#' In testing, cloud services (Box, Dropbox, etc.) do not work well as they seem to interfere with the syncing
+#' of apsim database files. It is suggested that they are turned off when running simulations.
+#' 
+#' In version 2.8.0 and earlier the original file was changed after it was edited by the function.
+#' In newer versions, the function creates a backup file and then restores it after the code is
+#' executed, if there are no errors.
 #'
 #' Suggested reading on the topic of sensitivity analysis:
 #'
@@ -59,7 +65,7 @@ sens_apsimx <- function(file, src.dir = ".",
                         convert,
                         replacement,
                         grid,
-                        soil.profiles,
+                        soil.profiles = NULL,
                         summary = c("mean", "max", "var", "sd", "none"),
                         root,
                         verbose = TRUE,
@@ -70,9 +76,18 @@ sens_apsimx <- function(file, src.dir = ".",
   if(missing(file))
     stop("file is missing with no default", call. = FALSE)
 
-  .check_apsim_name(file)
-  .check_apsim_name(src.dir)
+  if(isFALSE(get("allow.path.spaces", apsimx::apsimx.options))){
+    .check_apsim_name(file)
+    .check_apsim_name(normalizePath(src.dir))
+  }
 
+  ### Make a backup copy of the file, since it will be modified later
+  file.backup.name <- paste0(tools::file_path_sans_ext(file), "-backup.apsimx")
+  file.copy(from = file.path(src.dir, file), 
+            to = file.path(src.dir, file.backup.name))
+  if(verbose > 1L)
+    message("Created backup copy of original file")
+  
   if(cores > 1L){
 
     if(!requireNamespace("future", quietly = TRUE)){
@@ -82,19 +97,22 @@ sens_apsimx <- function(file, src.dir = ".",
 
     if(cores > (nrow(grid) - 1))
        stop("'cores' should be an integer smaller than the number of simulations minus one", call. = FALSE)
-    detect.cores <- parallel::detectCores()
+    ### detect.cores <- parallel::detectCores()
+    ### Suggestion by Henrik Bengtsson to use parallely instead of parallel (github issue #191)
+    ### Also, the code below might be unnecessary (or redundant) - I will keep it for now
+    detect.cores <- parallelly::availableCores()
     if(cores > detect.cores)
       stop("'cores' argument should not be higher than the number of available cores", call. = FALSE)
 
-    fp.op <- getOption('future.plan')
-
-    ### Maybe multisession needs to be the default regardless of platform
-    if(is.null(fp.op)){
-      oplan <- future::plan(strategy = "multisession", workers = cores)
-      on.exit(oplan)
-    }else{
-      future::plan(fp.op, workers = cores)
-    } 
+    ## This code was contributed by Henrik Bengtsson
+    cplan <- future::plan() ## This is the current plan
+    if (inherits(cplan, "sequential")) {
+      oplan <- future::plan(strategy = future::multisession, workers = cores)
+    } else {
+      oplan <- future::plan(cplan, workers = cores) 
+    }
+    on.exit(future::plan(oplan), add = TRUE) 
+    
   }
 
   ## This might offer suggestions in case there is a typo in 'file'
@@ -120,12 +138,12 @@ sens_apsimx <- function(file, src.dir = ".",
   if(missing(replacement)) replacement <- rep(FALSE, length(parm.paths))
 
   if(missing(grid))
-    stop("grid argument is missing")
+    stop("grid argument is missing", call. = FALSE)
 
   grid <- as.data.frame(grid)
 
   if(ncol(grid) != length(parm.paths))
-      stop("Number of columns in grid should be equal to the number of parameters")
+      stop("Number of columns in grid should be equal to the number of parameters", call. = FALSE)
 
   ## When verbose this is more compact than previous behavior
   pb <- utils::txtProgressBar(min = 1, max = nrow(grid), style = 3)
@@ -133,7 +151,8 @@ sens_apsimx <- function(file, src.dir = ".",
   ## Check that the name in the grid appears somewhere in the parameter path
   for(.ii in seq_along(parm.paths)){
     is.dot.present <- grepl(".", names(grid)[.ii], fixed = TRUE)
-    is.soil <- grepl("soil.profile|Soil$", names(grid)[.ii]) ## This should be of length one
+    is.soil <- grepl("soil.profile|^Soil$", names(grid)[.ii]) ## This should be of length one
+
     if(is.dot.present && !is.soil){
       ## The first element should match simulation names
       fspe <- strsplit(names(grid)[.ii], ".", fixed = TRUE)[[1]]
@@ -153,9 +172,11 @@ sens_apsimx <- function(file, src.dir = ".",
         warning("names in second element of grid object name do not match parameter path name")
       }
     }else{
-      if(grepl("soil.profile|Soil$", parm.paths[.ii])){
+      ### If the intention is to replace soil profiles
+      ### Then 'parm.path' can only be either 'soil.profile' or 'Soil'
+      if(grepl("soil.profile|^Soil$", parm.paths[.ii])){
        ### Check that there is a column in grid that corresponds to soils
-        isn <- grepl("soil.profile|Soil$", names(grid)[.ii], ignore.case = TRUE)
+        isn <- grepl("soil.profile|^Soil$", names(grid)[.ii], ignore.case = TRUE)
         if(isFALSE(isn)){
           cat("Name in grid:", names(grid)[.ii], "\n")
           cat("parameter name", parm.paths[.ii], "\n")
@@ -167,7 +188,7 @@ sens_apsimx <- function(file, src.dir = ".",
           cat("Name in grid:", names(grid)[.ii], "\n")
           cat("parameter name", parm.paths[.ii], "\n")
           warning("names in grid object do not match parameter path name")
-        }        
+        }
       }
     }
   }
@@ -217,7 +238,7 @@ sens_apsimx <- function(file, src.dir = ".",
         }
       }else{
         ## cat("parameter path", parm.paths[.j], "\n")
-        if(!grepl("soil.profile|Soil", parm.paths[.j])){
+        if(!grepl("soil.profile|Soil$", parm.paths[.j])){
           if(missing(root)){
             edit_apsimx(file = file,
                         src.dir = src.dir,
@@ -237,16 +258,16 @@ sens_apsimx <- function(file, src.dir = ".",
                         value = par.val,
                         root = root,
                         verbose = FALSE)
-          }          
+          }
         }else{
-         if(verbose) cat("Replacing the soil profile in simulation", .i, "\n")
+         if(verbose > 1L) cat("Replacing the soil profile in simulation", .i, "\n")
          wspc <- grep("soil.profile|Soil$", names(grid)) ## Which one is the soil profile column
-         
+
          if(length(wspc) == 0){
            cat("Result of wspc:", wspc, "\n")
            stop("'soil.profile' column not found in grid", call. = FALSE)
          }
-         
+
          ## There can be more than one column with soils that need to be replaced
          for(.k in seq_along(wspc)){
            if(grepl("Soil$", names(grid)[wspc[.k]])){
@@ -265,8 +286,8 @@ sens_apsimx <- function(file, src.dir = ".",
                                               src.dir = src.dir,
                                               wrt.dir = src.dir,
                                               soil.profile = soil.profiles[[soil.profile.index]],
-                                              overwrite = FALSE,
-                                              verbose = FALSE)           
+                                              overwrite = TRUE,
+                                              verbose = FALSE)
            }else{
              edit_apsimx_replace_soil_profile(file = file,
                                               src.dir = src.dir,
@@ -333,12 +354,18 @@ sens_apsimx <- function(file, src.dir = ".",
         ## to prevent conflicts with the file below
         file.to.run <- file.path(src.dir, paste0(tools::file_path_sans_ext(file),"-", core.counter, ".apsimx"))
 
-        file.copy(from = file.path(src.dir, file), to = file.to.run)
-
-        sim.list[[core.counter]] <- future::future(apsimx(file = paste0(tools::file_path_sans_ext(file), "-", core.counter, ".apsimx"),
+        fcs <- file.copy(from = file.path(src.dir, file), to = file.to.run)
+        if(isFALSE(fcs)){
+          message("Iteration:", .i)
+          message("Core counter:", core.counter)
+          message("File to run": file.to.run)
+          stop("Failed to copy file", call. = FALSE)
+        }
+          
+        sim.list[[core.counter]] <- future::future(try(apsimx(file = paste0(tools::file_path_sans_ext(file), "-", core.counter, ".apsimx"),
                                                                 src.dir = src.dir,
                                                                 silent = TRUE,
-                                                                cleanup = TRUE, value = "report"),
+                                                                cleanup = TRUE, value = "report"), silent = TRUE),
                                                    conditions = character(0))
         break
       }
@@ -355,7 +382,6 @@ sens_apsimx <- function(file, src.dir = ".",
 
       if(verbose > 1){
         print(sapply(sim.list, class))
-
         cat("Length sim.list:", length(sim.list), "\n")
         cat("Number of simulations remaining:", num.sim.rem, "\n")
       }
@@ -404,7 +430,12 @@ sens_apsimx <- function(file, src.dir = ".",
       }
 
       for(j in seq_len(length(simc))){
-        file.remove(file.path(src.dir, paste0(tools::file_path_sans_ext(file), "-", j, ".apsimx")))
+        frs <- file.remove(file.path(src.dir, paste0(tools::file_path_sans_ext(file), "-", j, ".apsimx")))
+        if(isFALSE(frs)){
+          message("Iteration:", .i)
+          message("Core counter:", core.counter)
+          stop("Failed to remove file", call. = FALSE)
+        }
       }
     }
 
@@ -465,6 +496,14 @@ sens_apsimx <- function(file, src.dir = ".",
     cdat <- col.sim
   }
 
+  ### If there are no errors, which usually happen earlier
+  file.remove(file.path(src.dir, file)) ## Remove the modified file
+  file.rename(from = file.path(src.dir, file.backup.name),
+            to = file.path(src.dir, file))
+  if(verbose > 1L){
+    message("Restoring the original file from backup")
+  }
+  
   attr(cdat, "summary") <- summary
 
   ans <- structure(list(grid.sims = cdat, grid = grid, parm.paths = parm.paths),
@@ -509,7 +548,7 @@ sens_summary <- function(sim, summary = c("mean", "max", "var", "sd", "none"),
 
 #' The argument \sQuote{formula} can be as in \code{\link{lm}}. The response
 #' can be omitted.
-#' 
+#'
 #' @rdname sens_apsimx
 #' @description Summary computes variance-based sensitivity indexes from an object of class \sQuote{sens_apsim}
 #' @param object object of class \sQuote{sens_apsim}
@@ -574,18 +613,18 @@ summary.sens_apsim <- function(object, ..., formula, scale = FALSE, select = "al
     }
 
     if(missing(formula) || formula == 1L){
-      frml <- paste("y ~", paste(names(X), collapse = "+"))  
+      frml <- paste("y ~", paste(names(X), collapse = "+"))
     }else{
       if(is.numeric(formula)){
         frml0 <- paste(names(X), collapse = "+")
         if(formula == 2L){
-          frml <- paste("y ~ (", paste(names(X), collapse = "+"), ")^2")  
+          frml <- paste("y ~ (", paste(names(X), collapse = "+"), ")^2")
         }
         if(formula == 3L){
-          frml <- paste("y ~ (", paste(names(X), collapse = "+"), ")^3")  
+          frml <- paste("y ~ (", paste(names(X), collapse = "+"), ")^3")
         }
         if(formula == 4L){
-          frml <- paste("y ~ (", paste(names(X), collapse = "+"), ")^4")  
+          frml <- paste("y ~ (", paste(names(X), collapse = "+"), ")^4")
         }
         if(formula != 2L && formula != 3L && formula != 4L)
           stop("When 'formula' is a number it needs to be either 2, 3 or 4", call. = FALSE)
@@ -594,11 +633,11 @@ summary.sens_apsim <- function(object, ..., formula, scale = FALSE, select = "al
           tt <- stats::terms(formula)
           frml <- stats::reformulate(attr(tt, "term.labels"), response = "y")
         }else{
-          frml <- formula                  
+          frml <- formula
         }
       }
     }
-    
+
     fit <- stats::lm(formula = frml, data = dat, na.action = "na.omit")
     if(inherits(fit, "try-error")) next
     w <- fit$weights
@@ -606,9 +645,9 @@ summary.sens_apsim <- function(object, ..., formula, scale = FALSE, select = "al
     mss <- sum(if (is.null(w)) fit$fitted.values^2 else w * fit$fitted.values^2)
     if(ssr < 1e-10 * mss){
       cat("Variable:", nms.resp.var[.i], "\n")
-      warning("ANOVA F-tests on an essentially perfect fit are unreliable", 
+      warning("ANOVA F-tests on an essentially perfect fit are unreliable",
               immediate. = TRUE)
-    } 
+    }
     sfit <- as.matrix(stats::anova(fit, ...))
     wrn <- warnings()
     kable.caption <- paste("Variable:", nms.resp.var[.i])
@@ -621,11 +660,11 @@ summary.sens_apsim <- function(object, ..., formula, scale = FALSE, select = "al
     pmatd <- pmatd[order(pmatd$SS, decreasing = TRUE),]
     if(verbose){
       print(knitr::kable(pmatd, caption = kable.caption, digits = 0))
-      cat("\n")      
+      cat("\n")
     }
 
     ansi <- data.frame(input = row.names(pmatd), SI = round(pmatd[, "SI (%)"], 1))
-    names(ansi) <- c("input", paste(nms.resp.var[.i], "SI (%)"))  
+    names(ansi) <- c("input", paste(nms.resp.var[.i], "SI (%)"))
     if(.j == 0){
       ans <- ansi
     }else{
@@ -635,7 +674,7 @@ summary.sens_apsim <- function(object, ..., formula, scale = FALSE, select = "al
     .j <- .j + 1
   }
   if(.j == 0) return("No variables reported. Are they all constant?")
-  
+
   ## Need to reorder (on average)
   rmns <- rowMeans(ans[, -1, drop = FALSE])
   ans <- ans[order(rmns, decreasing = TRUE), ]
@@ -652,7 +691,7 @@ summary.sens_apsim <- function(object, ..., formula, scale = FALSE, select = "al
 #' @export
 #'
 print.sens_apsim <- function(x, ..., variables = FALSE, summary = FALSE){
-  
+
   ### Print parameters in grid
   cat("Grid dimensions (rows columns):", dim(x$grid), "\n")
   ### Grid names
@@ -667,5 +706,5 @@ print.sens_apsim <- function(x, ..., variables = FALSE, summary = FALSE){
   if(variables) print(output.variables)
   ### Print data.frame summary
   if(summary) print(summary(x$grid.sims))
-  
-}
\ No newline at end of file
+
+}
