@@ -27,7 +27,10 @@ view_apsimx <- function(file, src.dir, viewer = c("json","react"), ...){
   
   if(missing(file)) stop("need to specify file name")
   
-  .check_apsim_name(file)
+  if(isFALSE(get("allow.path.spaces", apsimx::apsimx.options))){
+    .check_apsim_name(file)
+    .check_apsim_name(normalizePath(src.dir))
+  }
   
   ## The might offer suggestions in case there is a typo in 'file'
   file.names <- dir(path = src.dir, pattern = ".apsimx$", ignore.case=TRUE)
